package speiger.src.coreengine.math.vector.bytes;

import java.nio.ByteBuffer;

import speiger.src.coreengine.math.MathUtils;
import speiger.src.coreengine.math.vector.doubles.Vec4d;
import speiger.src.coreengine.math.vector.floats.Vec4f;
import speiger.src.coreengine.math.vector.ints.Vec4i;
import speiger.src.coreengine.math.vector.longs.Vec4l;
import speiger.src.coreengine.math.vector.shorts.Vec4s;

public interface Vec4b extends Vecb {
	public static final Vec4b ZERO = of();
	public static final Vec4b MINUS_ONE = of((byte)-1);
	public static final Vec4b ONE = of((byte)1);
	
	public static Vec4b mutable() { return new Vec4bMutable(); }
	public static Vec4b mutable(byte value) { return new Vec4bMutable(value); }
	public static Vec4b mutable(byte x, byte y, byte z, byte w) { return new Vec4bMutable(x, y, z, w); }
	public static Vec4b mutable(Vec4b vec) { return mutable(vec.x(), vec.y(), vec.z(), vec.w()); }
	public static Vec4b of() { return new Vec4bImmutable(); }
	public static Vec4b of(byte value) { return new Vec4bImmutable(value); }
	public static Vec4b of(byte x, byte y, byte z, byte w) { return new Vec4bImmutable(x, y, z, w); }
	public static Vec4b of(Vec4b vec) { return of(vec.x(), vec.y(), vec.z(), vec.w()); }
	
	public byte x();
	public byte y();
	public byte z();
	public byte w();
	public Vec4b x(byte x);
	public Vec4b y(byte y);
	public Vec4b z(byte z);
	public Vec4b w(byte w);
	
	@Override
	public default byte[] asArray() { return new byte[] {x(), y(), z(), w()}; }
	@Override
	public Vec4b copy();
	@Override
	public default Vec4b abs() { return set((byte)Math.abs(x()), (byte)Math.abs(y()), (byte)Math.abs(z()), (byte)Math.abs(w())); }
	@Override
	public default Vec4b negate() { return set((byte)0, (byte)0, (byte)0, (byte)0); }
	@Override
	public default Vec4b invert() { return set((byte)-x(), (byte)-y(), (byte)-z(), (byte)-w()); }
	@Override
	public default Vec4b add(byte value) { return add(value, value, value, value); }
	public default Vec4b add(Vec4b value) { return add(value.x(), value.y(), value.z(), value.w()); }
	public default Vec4b add(byte x, byte y, byte z, byte w) { return set((byte)(x() + x), (byte)(y() + y), (byte)(z() + z), (byte)(w() + w)); }
	@Override
	public default Vec4b sub(byte value) { return sub(value, value, value, value); }
	public default Vec4b sub(Vec4b value) { return sub(value.x(), value.y(), value.z(), value.w()); }
	public default Vec4b sub(byte x, byte y, byte z, byte w) { return set((byte)(x() - x), (byte)(y() - y), (byte)(z() - z), (byte)(w() - w)); }
	@Override
	public default Vec4b multiply(byte value) { return multiply(value, value, value, value); }
	public default Vec4b multiply(Vec4b value) { return multiply(value.x(), value.y(), value.z(), value.w()); }
	public default Vec4b multiply(byte x, byte y, byte z, byte w) { return set((byte)(x() * x), (byte)(y() * y), (byte)(z() * z), (byte)(w() * w)); }
	@Override
	public default Vec4b devide(byte value) { return devide(value, value, value, value); }
	public default Vec4b devide(Vec4b value) { return devide(value.x(), value.y(), value.z(), value.w()); }
	public default Vec4b devide(byte x, byte y, byte z, byte w) { return set((byte)(x() / x), (byte)(y() / y), (byte)(z() / z), (byte)(w() / w)); }
	@Override
	public default Vec4b set(byte value) { return set(value, value, value, value); }
	public default Vec4b set(Vec4b value) { return set(value.x(), value.y(), value.z(), value.w()); }
	public Vec4b set(byte x, byte y, byte z, byte w);
	public default double distanceTo(Vec4b value) { return distanceTo(value.x(), value.y(), value.z(), value.w()); }
	public default double distanceTo(byte x, byte y, byte z, byte w) {  return Math.sqrt(distanceTo(x, y, z, w)); } 
	public default long distanceToSquared(Vec4b value) { return distanceToSquared(value.x(), value.y(), value.z(), value.w()); }
	public default long distanceToSquared(byte x, byte y, byte z, byte w) {
		long xPos = x() - x;
		long yPos = y() - y;
		long zPos = z() - z;
		long wPos = w() - w;
		return (xPos * xPos) + (yPos * yPos) + (zPos * zPos) + (wPos * wPos);
	}
	
	@Override
	public default long lengthSquared() { return (x() * x()) + (y() * y()) + (z() * z()) + (w() * w()); }
	public default long dotProduct(Vec4b value) { return dotProduct(value.x(), value.y(), value.z(), value.w()); }
	public default long dotProduct(byte x, byte y, byte z, byte w) { return (x() * x) + (y() * y) + (z() * z) + (w() * w); };
	public default Vec4b min(Vec4b other) { return min(other, this); }
	public default Vec4b min(Vec4b other, Vec4b result) { return min(other.x(), other.y(), other.z(), other.w(), result); }
	public default Vec4b min(byte x, byte y, byte z, byte w) { return min(x, y, z, w, this); }
	public default Vec4b min(byte x, byte y, byte z, byte w, Vec4b result) { return result.set((byte)Math.min(x(), x), (byte)Math.min(y(), y), (byte)Math.min(z(), z), (byte)Math.min(w(), w)); }
	public default Vec4b max(Vec4b other) { return max(other, this); }
	public default Vec4b max(Vec4b other, Vec4b result) { return max(other.x(), other.y(), other.z(), other.w(), result); }
	public default Vec4b max(byte x, byte y, byte z, byte w) { return max(x, y, z, w, this); }
	public default Vec4b max(byte x, byte y, byte z, byte w, Vec4b result) { return result.set((byte)Math.max(x(), x), (byte)Math.max(y(), y), (byte)Math.max(z(), z), (byte)Math.max(z(), z)); }
	public default Vec4b difference(Vec4b other) { return difference(other, this); }
	public default Vec4b difference(Vec4b other, Vec4b result) { return difference(other.x(), other.y(), other.z(), other.w(), result); }
	public default Vec4b difference(byte x, byte y, byte z, byte w) { return difference(x, y, z, w, this); }
	public default Vec4b difference(byte x, byte y, byte z, byte w, Vec4b result) { return result.set((byte)(x() - x), (byte)(y() - y), (byte)(z() - z), (byte)(w() - w)); }
	@Override
	public default Vec4b clamp(byte min, byte max) { return clamp(min, max, ALL); }
	public default Vec4b clamp(byte min, byte max, Vec4b result) { return clamp(min, max, result, ALL); }
	@Override
	public default Vec4b clamp(byte min, byte max, int filter) { return clamp(min, max, this, filter); }
	public default Vec4b clamp(byte min, byte max, Vec4b result, int filter) { return result.set((filter & X) == 0 ? x() : MathUtils.clamp(min, max, x()), (filter & Y) == 0 ? y() : MathUtils.clamp(min, max, y()), (filter & Z) == 0 ? z() : MathUtils.clamp(min, max, z()), (filter & W) == 0 ? w() : MathUtils.clamp(min, max, w())); }
	@Override
	public default Vec4b store(ByteBuffer buffer) {
		buffer.put(x()).put(y()).put(z()).put(w());
		return this;
	}
	@Override
	public default Vec4b load(ByteBuffer buffer) { return set(buffer.get(), buffer.get(), buffer.get(), buffer.get()); }
	@Override
	public default Vec4s asShort() { return isMutable() ? Vec4s.mutable(x(), y(), z(), w()) : Vec4s.of(x(), y(), z(), w()); }
	@Override
	public default Vec4i asInt() { return isMutable() ? Vec4i.mutable(x(), y(), z(), w()) : Vec4i.of(x(), y(), z(), w()); }
	@Override
	public default Vec4l asLong() { return isMutable() ? Vec4l.mutable(x(), y(), z(), w()) : Vec4l.of(x(), y(), z(), w()); }
	@Override
	public default Vec4f asFloat() { return isMutable() ? Vec4f.mutable(x(), y(), z(), w()) : Vec4f.of(x(), y(), z(), w()); }
	@Override
	public default Vec4d asDouble() {return isMutable() ? Vec4d.mutable(x(), y(), z(), w()) : Vec4d.of(x(), y(), z(), w()); }
	@Override
	public default Vec4b asMutable() { return isMutable() ? this : mutable(this); }
	@Override
	public default Vec4b asImmutable() { return isMutable() ? of(this) : this; }
	@Override
	public default Vec4b copyAsMutable() { return mutable(this); }
	@Override
	public default Vec4b copyAsImmutable() { return of(this); }
}
